unit _AxlGen;

interface

uses
	Windows,
	Messages,
	SysUtils,
	Classes,
	Graphics,
	Controls,
	Forms,
	Dialogs,
	DsgnIntf,
	StdCtrls,
	ComCtrls,
	ExtCtrls,
	ToolWin,
	Buttons,
	Menus,
	ImgList,
	ShellAPI
{$IFDEF AxlGenTest}
	,_Test
{$ENDIF}
	;

type
	TAddComponentEvent = procedure(Sender: TObject; Level: Integer) of object;

	TAcceleratorGeneratorForm = class(TForm)
    sbAcceleratorGenerator: TStatusBar;
    tbAcceleratorGenerator: TToolBar;
    bAssign: TSpeedButton;
    ilDummy: TImageList;
		bFind: TSpeedButton;
    bMoveDown: TSpeedButton;
		bMoveUp: TSpeedButton;
		bClearAll: TSpeedButton;
		bClear: TSpeedButton;
		pmAccelleratorGenerator: TPopupMenu;
		miNothing: TMenuItem;
    miCaption: TMenuItem;
    miCaptionLength: TMenuItem;
		miUniqueLetters: TMenuItem;
		itComponentGraphics: TImageList;
    bAbout: TSpeedButton;
		nbAbout: TNotebook;
    tvControls: TTreeView;
    pHolder: TPanel;
    pAbout: TPanel;
    lTitle: TLabel;
		lVersion: TLabel;
    lCopyright: TLabel;
		lDevelopedBy: TLabel;
    lSomeDevelopers: TLabel;
    lInformation: TLabel;
    lHomepage: TLabel;
		procedure FormShow(Sender: TObject);
		procedure lvControlsClick(Sender: TObject);
		procedure bClearAllClick(Sender: TObject);
    procedure bClearClick(Sender: TObject);
		procedure SortBy(Sender: TObject);
		procedure lvControlsCompare(Sender: TObject; Item1, Item2: TListItem;
			Data: Integer; var Compare: Integer);
		procedure bAssignClick(Sender: TObject);
		procedure bFindClick(Sender: TObject);
		procedure lvControlsEdited(Sender: TObject; Item: TListItem;
			var S: String);
		procedure bMoveUpClick(Sender: TObject);
		procedure bMoveDownClick(Sender: TObject);
		procedure ShowHint(Sender: TObject; Shift: TShiftState; X,
			Y: Integer);
		procedure tvControlsCompare(Sender: TObject; Node1, Node2: TTreeNode;
			Data: Integer; var Compare: Integer);
		procedure bAboutClick(Sender: TObject);
		procedure FormResize(Sender: TObject);
		procedure tvControlsClick(Sender: TObject);
    procedure lHomepageClick(Sender: TObject);
    procedure tvControlsChange(Sender: TObject; Node: TTreeNode);
	private
		{ Private declarations }
		fForm: TCustomForm;

		procedure UpdateButtons;
		function GetUniqueLetterCount(ACaption: string): Integer;
		function GetAccelerator(ACaption: string): string;
		function HasAccelerator(ACaption: string): Boolean;
		function AcceleratorLength(ACaption: string): Integer;
		function DeleteAccelerator(ACaption: string): string;
		function WordCount(ACaption: string): Integer;
		function GetWord(ACaption: string; AWordIndex: Integer): string;
		procedure EnumComponents(AComponent: TComponent; OnFoundComponent: TAddComponentEvent);
		procedure AddComponent(Sender: TObject; Level: Integer);
		procedure ResizeAbout;
		procedure Find(AParentUsed: TStringList; AParent: TComponent; ATreeNode: TTreeNode);
	public
		{ Public declarations }
		property Form: TCustomForm read fForm write fForm;
	end;

	TAcceleratorGeneratorEditor = class(TDefaultEditor)
	private
		fAcceleratorGeneratorForm: TAcceleratorGeneratorForm;
	public
{$ifdef VER120}
		constructor Create(AComponent: TComponent; ADesigner: IFormDesigner); override;
{$else}
		constructor Create(AComponent: TComponent; ADesigner: TFormDesigner); override;
{$endif}
		destructor Destroy; override;

		procedure ExecuteVerb(Index: Integer); override;
		function GetVerb(Index: Integer): string; override;
		function GetVerbCount: Integer; override;
		procedure Edit; override;
	end;

var
	AcceleratorGeneratorForm: TAcceleratorGeneratorForm;

procedure Register;

implementation

{$R *.DFM}

{$ifdef VER120}
constructor TAcceleratorGeneratorEditor.Create(AComponent: TComponent; ADesigner: IFormDesigner); 
{$else}
constructor TAcceleratorGeneratorEditor.Create(AComponent: TComponent; ADesigner: TFormDesigner);
{$endif}                                      
begin
	inherited Create(AComponent,ADesigner);

	fAcceleratorGeneratorForm := nil;
end;

destructor TAcceleratorGeneratorEditor.Destroy;
begin
	fAcceleratorGeneratorForm.Free;

	inherited Destroy;
end;

procedure TAcceleratorGeneratorEditor.ExecuteVerb(Index: Integer);
begin
	if Index <> 0 then
		Exit;
	Edit;
end;

function TAcceleratorGeneratorEditor.GetVerb(Index: Integer): AnsiString;
begin
	Result := '&Accelerator Generator...';
end;

function TAcceleratorGeneratorEditor.GetVerbCount: Integer;
begin
	Result := 1;
end;

procedure TAcceleratorGeneratorEditor.Edit;
var
	C: TComponent;
begin
	if not Assigned(fAcceleratorGeneratorForm) then
		fAcceleratorGeneratorForm := TAcceleratorGeneratorForm.Create(nil);

	C := Component;
	while not (C is TCustomForm) do
		C := TControl(C).Parent;

	fAcceleratorGeneratorForm.Form := TCustomForm(C);
	fAcceleratorGeneratorForm.Show;
end;

procedure TAcceleratorGeneratorForm.EnumComponents(AComponent: TComponent; OnFoundComponent: TAddComponentEvent);
var
	I,J,Level: Integer;
	Done: Boolean;

	function GetLevel(AControl: TComponent): Integer;
	var
		C: TControl;
		M: TMenuItem;
	begin
		Result := -1;
		if AControl is TMainMenu then
			Result := 1;
		if (AControl is TControl) then
		begin
			Result := 0;
			C := TControl(AControl);
			while (C <> fForm) and (C <> nil) do
			begin
				Inc(Result);
				C := C.Parent;
			end;
		end;
		if (AControl is TMenuItem) then
		begin
			Result := -1;
			M := TMenuItem(AControl);
			while (M <> nil) do
			begin
				Inc(Result);
				M := M.Parent;
			end;
		end;
	end;

begin
	J := 1;
	repeat
		Done := True;
		for I := 0 to AComponent.ComponentCount-1 do
		begin
			if AComponent.Components[I].ClassType = TObject then
				J := 2;
			Level := GetLevel(AComponent.Components[I]);
			if J = Level then
			begin
				Done := False;
				OnFoundComponent(AComponent.Components[I],Level);
			end;
		end;
		Inc(J);
	until Done;
end;

procedure TAcceleratorGeneratorForm.FormShow(Sender: TObject);
begin
{$IFDEF AxlGenTest}
	fForm := TestForm;
{$ENDIF}
	nbAbout.PageIndex := 0;
	Caption := 'Accelerator Generator - ' + fForm.Name + ' (' + fForm.Caption + ')';
	tvControls.Items.Clear;
	EnumComponents(fForm,AddComponent);

	UpdateButtons;
	SortBy(nil);
end;

procedure Register;
begin
	RegisterComponentEditor(TForm, TAcceleratorGeneratorEditor);
	RegisterComponentEditor(TPanel, TAcceleratorGeneratorEditor);
end;

procedure TAcceleratorGeneratorForm.lvControlsClick(Sender: TObject);
begin
	UpdateButtons;
end;

function TAcceleratorGeneratorForm.GetUniqueLetterCount(ACaption: string): Integer;
var
	Used: TStringList;
	I: Integer;
	S: string;
begin
	Used := TStringList.Create;
	try
		for I := 1 to Length(ACaption) do
		begin
			S := '' + ACaption[I];
			if (S[1] < #65) or (S[1] > #128) then
					Continue;
			if Used.IndexOf(S) = -1 then
				Used.Add(S);
		end;

		Result := Used.Count;
	finally
		Used.Free;
	end;
end;

function TAcceleratorGeneratorForm.GetAccelerator(ACaption: string): string;
var
	I,
	Index: Integer;
begin
	if ACaption = '' then
	begin
		Result := '';
		Exit;
	end;
	Index := -1;
	for I := 1 to Length(ACaption)-1 do
		if (ACaption[I] = '&') and (ACaption[I+1] <> '&') then
			Index := I;
	Result := '' + ACaption[Index+1];
end;

function TAcceleratorGeneratorForm.HasAccelerator(ACaption: string): Boolean;
begin
	if ACaption = '' then
		Result := False
	else if ACaption = DeleteAccelerator(ACaption) then
		Result := False
	else
		Result := True;
end;

function TAcceleratorGeneratorForm.AcceleratorLength(ACaption: string): Integer;
begin
	Result := Length(DeleteAccelerator(ACaption));
end;

function TAcceleratorGeneratorForm.DeleteAccelerator(ACaption: string): string;
begin
	Result := ACaption;
	repeat
		if (Pos('&',Result) > Length(Result)) or
			(Pos('&',Result) = 0) then
			Break;
		if Result[Pos('&',Result)+1] <> '&' then
			Delete(Result,Pos('&',Result),1);
	until False;
end;

procedure TAcceleratorGeneratorForm.UpdateButtons;
begin
	if tvControls.Selected <> nil then
		bClear.Enabled := True
	else
		bClear.Enabled := False;

	bMoveUp.Enabled := False;
	bMoveDown.Enabled := False;
	if (miNothing.Checked = True) and Assigned(tvControls.Selected) then
	begin
		if tvControls.Selected.GetPrevSibling <> nil then
			bMoveUp.Enabled := True;
		if tvControls.Selected.GetNextSibling <> nil then
			bMoveDown.Enabled := True;
	end;
end;

procedure TAcceleratorGeneratorForm.bClearAllClick(Sender: TObject);
var
	I: Integer;
begin
	for I := 0 to tvControls.Items.Count-1 do
		tvControls.Items[I].Text := DeleteAccelerator(tvControls.Items[I].Text);
end;

procedure TAcceleratorGeneratorForm.bClearClick(Sender: TObject);
var
	I: Integer;
begin
	for I := 0 to tvControls.Items.Count-1 do
		if tvControls.Items[I].Selected then
			tvControls.Items[I].Text := DeleteAccelerator(tvControls.Items[I].Text);
end;

procedure TAcceleratorGeneratorForm.SortBy(Sender: TObject);
begin
	if Assigned(Sender) then
		TMenuItem(Sender).Checked := True;
	tvControls.AlphaSort;
	UpdateButtons;
end;

procedure TAcceleratorGeneratorForm.lvControlsCompare(Sender: TObject;
	Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
begin
	if miCaption.Checked then
	begin
		if DeleteAccelerator(Item1.Caption) < DeleteAccelerator(Item2.Caption) then
			Compare := -1
		else if DeleteAccelerator(Item1.Caption) > DeleteAccelerator(Item2.Caption) then
			Compare := 1
		else
			Compare := 0;
	end;
	if miCaptionLength.Checked then
	begin
		if AcceleratorLength(Item1.Caption) < AcceleratorLength(Item2.Caption) then
			Compare := -1
		else if AcceleratorLength(Item1.Caption) > AcceleratorLength(Item2.Caption) then
			Compare := 1
		else if DeleteAccelerator(Item1.Caption) < DeleteAccelerator(Item2.Caption) then
			Compare := -1
		else if DeleteAccelerator(Item1.Caption) > DeleteAccelerator(Item2.Caption) then
			Compare := 1
		else
			Compare := 0;
	end;
	if miUniqueLetters.Checked then
	begin
		if GetUniqueLetterCount(Item1.Caption) < GetUniqueLetterCount(Item2.Caption) then
			Compare := -1
		else if GetUniqueLetterCount(Item1.Caption) > GetUniqueLetterCount(Item2.Caption) then
			Compare := 1
		else if DeleteAccelerator(Item1.Caption) < DeleteAccelerator(Item2.Caption) then
			Compare := -1
		else if DeleteAccelerator(Item1.Caption) > DeleteAccelerator(Item2.Caption) then
			Compare := 1
		else
			Compare := 0;
	end;
end;

procedure TAcceleratorGeneratorForm.Find(AParentUsed: TStringList; AParent: TComponent; ATreeNode: TTreeNode);
var
	I,J,K,L: Integer;
	Used: TStringList;
	S,S1: string;
	Found,
	Done,
	AcceleratorRequired: Boolean;

	function GetLevel(ATreeNode: TTreeNode): Integer;
	begin
		Result := 0;
		while ATreeNode <> nil do
		begin
			Inc(Result);
			ATreeNode := ATreeNode.Parent;
		end;
	end;

begin
	Used := TStringList.Create;
	try
		if not (AParent is TMenuItem) then
			for I := 0 to AParentUsed.Count-1 do
				Used.Add(AParentUsed[I]);

		Done := True;
		for I := 0 to tvControls.Items.Count-1 do
		begin
			if tvControls.Items[I].Parent = ATreeNode then
			begin
				Done := False;
				S := tvControls.Items[I].Text;
				AcceleratorRequired := True;
				if TObject(tvControls.Items[I].Data) is TButton then
					if TButton(tvControls.Items[I].Data).Cancel or
						TButton(tvControls.Items[I].Data).Default then
						AcceleratorRequired := False;
				if (not HasAccelerator(S)) and AcceleratorRequired then
				begin
					L := 0;
					for J := 1 to WordCount(S) do
						if Length(GetWord(S,J)) > L then
							L := Length(GetWord(S,J));
					if TObject(tvControls.Items[I].Data) is TMenuItem then
					begin
						if (TMenuItem(tvControls.Items[I].Data).Caption = 'Exit') and
							(Used.IndexOf('x') = -1) then
						begin
							Used.Add('x');
							S := 'E&xit';
							Found := True;
						end;
					end
					else
						Found := False;
					if not Found then
						for K := 1 to L do
						begin
							for J := 1 to WordCount(S) do
							begin
								if K > Length(GetWord(S,J)) then
									Continue;
								S1 := GetWord(S,J)[K];
								if (S1[1] < #65) or (S1[1] > #128) then
									Continue;
								if Used.IndexOf(S1) = -1 then
								begin
									Found := True;
									Insert('&',S,Pos(S1,S));
									Used.Add(S1);
									Break;
								end;
							end;
							if Found then
								Break;
						end;
					if Found then
						tvControls.Items[I].Text := S;
				end
				else
					Used.Add(GetAccelerator(S));
			end;
		end;

		if not Done then
			for I := 0 to tvControls.Items.Count-1 do
				if tvControls.Items[I].Parent = ATreeNode then
					Find(Used,tvControls.Items[I].Data,tvControls.Items[I]);
	finally
		Used.Free;
	end;
end;

procedure TAcceleratorGeneratorForm.bFindClick(Sender: TObject);
var
	Used: TStringList;
begin
	Used := TStringList.Create;
	try
		Find(Used,nil,nil);
	finally
		Used.Free;
	end;
end;

procedure TAcceleratorGeneratorForm.lvControlsEdited(Sender: TObject;
	Item: TListItem; var S: String);
{var
	I: Integer;
	Used: TStringList;}
begin
{	if fHasAccelerator(S) then
		Item.SubItems[1] := 'No'
	else
		Item.SubItems[1] := 'Yes';

	Used := TStringList.Create;
	try
		for I := 0 to lvControls.Items.Count-1 do
		begin
			if lvControls.Items[I] = Item then
				Break;
			Used.Add(fGetAccelerator(lvControls.Items[I].Caption));
		end;

		if Used.IndexOf(fGetAccelerator(S)) <> -1 then
			S := fDeleteAccelerator(S);
	finally
		Used.Free;
	end;}
end;

function TAcceleratorGeneratorForm.WordCount(ACaption: string): Integer;
var
	I,
	Mode: Integer;
begin
	Mode := 0;
	Result := 0;
	if ACaption = '' then
		Exit;

	if not ((ACaption[1] = ' ') or ((ACaption[1] = #9)) or ((ACaption[1] = #10)) or ((ACaption[1] = #13))) then
		Mode := 1;

	Result := 0;
	I := 1;
	while I <= Length(ACaption) do
	begin
		case Mode of
			0:
				begin
					if not ((ACaption[I] = ' ') or ((ACaption[I] = #9)) or ((ACaption[I] = #10)) or ((ACaption[I] = #13))) then
					begin
						Mode := 1;
						Dec(I);
					end;
				end;
			1:
				begin
					Inc(Result);
					Mode := 2;
					Dec(I);
				end;
			2:
				begin
					if ((ACaption[I] = ' ') or ((ACaption[I] = #9)) or ((ACaption[I] = #10)) or ((ACaption[I] = #13))) then
					begin
						Mode := 0;
						Dec(I);
					end;
				end;
		end;
		Inc(I);
	end;
end;

function TAcceleratorGeneratorForm.GetWord(ACaption: string; AWordIndex: Integer): string;
var
	I,
	WordCount,
	Mode: Integer;
	Capture: Boolean;
begin
	Mode := 0;
	if ACaption = '' then
		Exit;

	if not ((ACaption[1] = ' ') or ((ACaption[1] = #9)) or ((ACaption[1] = #10)) or ((ACaption[1] = #13))) then
		Mode := 1;

	Capture := False;
	WordCount := 0;
	Result := '';
	I := 1;
	while I <= Length(ACaption) do
	begin
		case Mode of
			0:
				begin
					if not ((ACaption[I] = ' ') or ((ACaption[I] = #9)) or ((ACaption[I] = #10)) or ((ACaption[I] = #13))) then
					begin
						Mode := 1;
						Dec(I);
					end;
				end;
			1:
				begin
					Inc(WordCount);
					if WordCount = AWordIndex then
						Capture := True;
					Mode := 2;
					Dec(I);
				end;
			2:
				begin
					if ((ACaption[I] = ' ') or ((ACaption[I] = #9)) or ((ACaption[I] = #10)) or ((ACaption[I] = #13))) then
					begin
						Mode := 0;
						Capture := False;
					end;
					if Capture then
						Result := Result + ACaption[I];
				end;
		end;
		Inc(I);
	end;
end;

procedure TAcceleratorGeneratorForm.bMoveUpClick(Sender: TObject);
var
	T: TTreeNode;
begin
	tvControls.Items.BeginUpdate;
	T := tvControls.Selected.GetPrevSibling;
	if T.GetPrevSibling <> nil then
		tvControls.Selected.MoveTo(T,naInsert)
	else
		tvControls.Selected.MoveTo(T,naAddFirst);

	tvControls.Items.EndUpdate;
	UpdateButtons;
end;

procedure TAcceleratorGeneratorForm.bMoveDownClick(Sender: TObject);
var
	T: TTreeNode;
begin
	tvControls.Items.BeginUpdate;
	T := tvControls.Selected.GetNextSibling;
	if tvControls.Selected.GetPrevSibling <> nil then
		T.MoveTo(tvControls.Selected,naInsert)
	else
		T.MoveTo(tvControls.Selected,naAddFirst);
	tvControls.Items.EndUpdate;
	UpdateButtons;
end;

procedure TAcceleratorGeneratorForm.ShowHint(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
	sbAcceleratorGenerator.SimpleText := TWinControl(Sender).Hint;
end;

procedure TAcceleratorGeneratorForm.AddComponent(Sender: TObject; Level: Integer);
var
	Caption: string;
	I: Integer;
	TreeNode: TTreeNode;
	ImageIndex: Integer;

	function InParents(AControl,AParentControl: TControl): Boolean;
	var
		C: TComponent;
	begin
		Result := False;
		if not (AControl is TControl) then
			Exit;
		C := TComponent(AControl);
		if C is TControl then
		begin
			while (C <> AParentControl) and (C <> nil) do
				C := TControl(C).Parent;
			if C = AParentControl then
				Result := True;
		end;
		if C is TMenuItem then
		begin
			while (C <> AParentControl) and (C <> nil) do
				C := TMenuItem(C).Parent;
			if C = AParentControl then
				Result := True;
		end;
	end;

begin
	Caption := '';
	ImageIndex := -1;
	if Sender is TBitBtn then
	begin
		Caption := TBitBtn(Sender).Caption;
		ImageIndex := 0;
	end;
	if Sender is TButton then
	begin
		Caption := TButton(Sender).Caption;
		ImageIndex := 1;
	end;
	if Sender is TCheckBox then
	begin
		Caption := TCheckBox(Sender).Caption;
		ImageIndex := 2;
	end;
	if Sender is TGroupBox then
	begin
		Caption := TGroupBox(Sender).Caption;
		ImageIndex := 3;
	end;
	if Sender is TLabel then
		if Assigned(TLabel(Sender).FocusControl) then
		begin
			Caption := TLabel(Sender).Caption;
			ImageIndex := 4;
		end;
	if Sender is TRadioButton then
	begin
		Caption := TRadioButton(Sender).Caption;
		ImageIndex := 5;
	end;
	if Sender is TSpeedButton then
	begin
		Caption := TSpeedButton(Sender).Caption;
		ImageIndex := 6;
	end;
	if Sender is TStaticText then
		if Assigned(TStaticText(Sender).FocusControl) then
		begin
			Caption := TStaticText(Sender).Caption;
			ImageIndex := 7;
		end;
	if Sender is TTabSheet then
	begin
		Caption := TTabSheet(Sender).Caption;
		ImageIndex := 8;
	end;
	if Sender is TMenuItem then
	begin
		Caption := TMenuItem(Sender).Caption;
		ImageIndex := 9;
	end;

	if Caption <> '' then
	begin
		TreeNode := nil;
		for I := 0 to tvControls.Items.Count-1 do
			if InParents(TControl(Sender),tvControls.Items[I].Data) then
			begin
				TreeNode := tvControls.Items[I];
				Break;
			end;
		if Assigned(TreeNode) then
			TreeNode := tvControls.Items.AddChildObject(TreeNode,Caption,Sender)
		else
			TreeNode := tvControls.Items.AddObject(TreeNode,Caption,Sender);
		TreeNode.ImageIndex := ImageIndex;
		TreeNode.SelectedIndex := ImageIndex;
	end;
end;

procedure TAcceleratorGeneratorForm.bAssignClick(Sender: TObject);
var
	I: Integer;
begin
	for I := 0 to tvControls.Items.Count-1 do
	begin
		if TObject(tvControls.Items[I].Data) is TBitBtn then
			TBitBtn(tvControls.Items[I].Data).Caption := tvControls.Items[I].Text;
		if TObject(tvControls.Items[I].Data) is TButton then
			TButton(tvControls.Items[I].Data).Caption := tvControls.Items[I].Text;
		if TObject(tvControls.Items[I].Data) is TCheckBox then
			TCheckBox(tvControls.Items[I].Data).Caption := tvControls.Items[I].Text;
		if TObject(tvControls.Items[I].Data) is TGroupBox then
			TGroupBox(tvControls.Items[I].Data).Caption := tvControls.Items[I].Text;
		if TObject(tvControls.Items[I].Data) is TLabel then
			TLabel(tvControls.Items[I].Data).Caption := tvControls.Items[I].Text;
		if TObject(tvControls.Items[I].Data) is TRadioButton then
			TRadioButton(tvControls.Items[I].Data).Caption := tvControls.Items[I].Text;
		if TObject(tvControls.Items[I].Data) is TSpeedButton then
			TSpeedButton(tvControls.Items[I].Data).Caption := tvControls.Items[I].Text;
		if TObject(tvControls.Items[I].Data) is TStaticText then
			TStaticText(tvControls.Items[I].Data).Caption := tvControls.Items[I].Text;
		if TObject(tvControls.Items[I].Data) is TTabSheet then
			TTabSheet(tvControls.Items[I].Data).Caption := tvControls.Items[I].Text;
		if TObject(tvControls.Items[I].Data) is TMenuItem then
			TMenuItem(tvControls.Items[I].Data).Caption := tvControls.Items[I].Text;
	end;
end;

procedure TAcceleratorGeneratorForm.tvControlsCompare(Sender: TObject;
	Node1, Node2: TTreeNode; Data: Integer; var Compare: Integer);
begin
	if miCaption.Checked then
	begin
		if DeleteAccelerator(Node1.Text) < DeleteAccelerator(Node2.Text) then
			Compare := -1
		else if DeleteAccelerator(Node1.Text) > DeleteAccelerator(Node2.Text) then
			Compare := 1
		else
			Compare := 0;
	end;
	if miCaptionLength.Checked then
	begin
		if AcceleratorLength(Node1.Text) < AcceleratorLength(Node2.Text) then
			Compare := -1
		else if AcceleratorLength(Node1.Text) > AcceleratorLength(Node2.Text) then
			Compare := 1
		else if DeleteAccelerator(Node1.Text) < DeleteAccelerator(Node2.Text) then
			Compare := -1
		else if DeleteAccelerator(Node1.Text) > DeleteAccelerator(Node2.Text) then
			Compare := 1
		else
			Compare := 0;
	end;
	if miUniqueLetters.Checked then
	begin
		if GetUniqueLetterCount(Node1.Text) < GetUniqueLetterCount(Node2.Text) then
			Compare := -1
		else if GetUniqueLetterCount(Node1.Text) > GetUniqueLetterCount(Node2.Text) then
			Compare := 1
		else if DeleteAccelerator(Node1.Text) < DeleteAccelerator(Node2.Text) then
			Compare := -1
		else if DeleteAccelerator(Node1.Text) > DeleteAccelerator(Node2.Text) then
			Compare := 1
		else
			Compare := 0;
	end;
end;

procedure TAcceleratorGeneratorForm.ResizeAbout;
begin
	lTitle.Width := pAbout.Width - 4;
	lCopyright.Width := pAbout.Width - 4;
	lVersion.Width := pAbout.Width - 4;
	lDevelopedBy.Width := pAbout.Width - 4;
	lSomeDevelopers.Width := pAbout.Width - 4;
	lInformation.Width := pAbout.Width - 4;
	lInformation.Height := pAbout.Height - 4 - lInformation.Top;
	lHomepage.Width := pAbout.Width - 4;
end;

procedure TAcceleratorGeneratorForm.bAboutClick(Sender: TObject);
begin
	ResizeAbout;
	nbAbout.PageIndex := 1-nbAbout.PageIndex;
end;

procedure TAcceleratorGeneratorForm.FormResize(Sender: TObject);
begin
	ResizeAbout;
end;

procedure TAcceleratorGeneratorForm.tvControlsClick(Sender: TObject);
begin
	UpdateButtons;
end;

procedure TAcceleratorGeneratorForm.lHomepageClick(Sender: TObject);
var
	Filename: string;
begin
	Filename := lHomepage.Caption;
	ShellExecute(Application.Handle,'open',PChar(Filename),nil,nil,SW_NORMAL);
end;

procedure TAcceleratorGeneratorForm.tvControlsChange(Sender: TObject;
  Node: TTreeNode);
begin
  UpdateButtons;
end;

end.


